/* eslint-env mocha */
/* eslint-disable no-unused-expressions */
const { expect } = require('chai');
const FfmpegMediaFile = require('./FfmpegMediaFile');
const { inputQuicktimePath } = require('../../test/fixtures');
require('dotenv').config();

describe('FfmpegMediaFile', () => {
  describe('Methods mapping to ffmpeg options', () => {
    describe('an()', () => {
      it('adds an to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.an();
        expect(test.hasOption('an')).to.be.true;
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.an();
        expect(test2).to.equal(test);
      });
    });

    describe('bitexact()', () => {
      it('adds bitexact to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.bitexact();
        expect(test.hasOption('bitexact')).to.be.true;
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.bitexact();
        expect(test2).to.equal(test);
      });
    });

    describe('dn()', () => {
      it('adds dn to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.dn();
        expect(test.hasOption('dn')).to.be.true;
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.dn();
        expect(test2).to.equal(test);
      });
    });

    describe('f()', () => {
      it('adds f to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.f('mov');
        expect(test.hasOption('f')).to.be.true;
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.f('mov');
        expect(test2).to.equal(test);
      });
    });

    describe('ss()', () => {
      it('adds ss to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: 60 });
      });

      it('adds ss to options with a float argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss(34.67);
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: 34.67 });
      });

      it('adds ss to options with an integer stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss('60');
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: 60 });
      });

      it('adds ss to options with a float stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss('34.67');
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: 34.67 });
      });

      it('adds ss to options with an mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss('25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: '25:15' });
      });

      it('adds ss to options with an h:mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss('0:25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: '0:25:15' });
      });

      it('adds ss to options with an hh:mm:ss.mm argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss('0:25:15.125');
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: '0:25:15.125' });
      });

      it('throws when called with an invalid time string', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        expect(() => test.ss('1:5.6.7')).to.throw();
      });

      it('adds ss to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.ss(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ ss: 60 });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.f('mov');
        expect(test2).to.equal(test);
      });
    });

    describe('t()', () => {
      it('adds t to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ t: 60 });
      });

      it('adds t to options with a float argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t(34.67);
        expect(test.ffmpegSettings.options).to.deep.equal({ t: 34.67 });
      });

      it('adds t to options with an integer stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t('60');
        expect(test.ffmpegSettings.options).to.deep.equal({ t: 60 });
      });

      it('adds t to options with a float stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t('34.67');
        expect(test.ffmpegSettings.options).to.deep.equal({ t: 34.67 });
      });

      it('adds t to options with an mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t('25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ t: '25:15' });
      });

      it('adds t to options with an h:mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t('0:25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ t: '0:25:15' });
      });

      it('adds t to options with an hh:mm:ss.mm argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t('0:25:15.125');
        expect(test.ffmpegSettings.options).to.deep.equal({ t: '0:25:15.125' });
      });

      it('throws when called with an invalid time string', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        expect(() => test.t('1:5.6.7')).to.throw();
      });

      it('adds t to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.t(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ t: 60 });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.t(60);
        expect(test2).to.equal(test);
      });
    });

    describe('to()', () => {
      it('adds to to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ to: 60 });
      });

      it('adds to to options with a float argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to(34.67);
        expect(test.ffmpegSettings.options).to.deep.equal({ to: 34.67 });
      });

      it('adds to to options with an integer stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to('60');
        expect(test.ffmpegSettings.options).to.deep.equal({ to: 60 });
      });

      it('adds to to options with a float stored as string argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to('34.67');
        expect(test.ffmpegSettings.options).to.deep.equal({ to: 34.67 });
      });

      it('adds to to options with an mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to('25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ to: '25:15' });
      });

      it('adds to to options with an h:mm:ss argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to('0:25:15');
        expect(test.ffmpegSettings.options).to.deep.equal({ to: '0:25:15' });
      });

      it('adds to to options with an hh:mm:ss.mm argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to('0:25:15.125');
        expect(test.ffmpegSettings.options).to.deep.equal({ to: '0:25:15.125' });
      });

      it('throws when called with an invalid time string', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        expect(() => test.to('1:5.6.7')).to.throw();
      });

      it('adds to to options with an integer argument', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.to(60);
        expect(test.ffmpegSettings.options).to.deep.equal({ to: 60 });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.f('mov');
        expect(test2).to.equal(test);
      });
    });

    describe('sn()', () => {
      it('adds dn to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.sn();
        expect(test.ffmpegSettings.options).to.deep.equal({ sn: null });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.sn();
        expect(test2).to.equal(test);
      });
    });

    describe('sws_flags()', () => {
      it('adds c:a to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.sws_flags('fast_bilinear');
        expect(test.ffmpegSettings.options).to.deep.equal({ sws_flags: 'fast_bilinear' });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.sws_flags('fast_bilinear');
        expect(test2).to.equal(test);
      });
    });

    describe('vn()', () => {
      it('adds dn to options', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.vn();
        expect(test.ffmpegSettings.options).to.deep.equal({ vn: null });
      });

      it('returns this', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        const test2 = test.vn();
        expect(test2).to.equal(test);
      });
    });
  });

  describe('Integration Tests', () => {
    describe('addOption()', () => {
      it('adds an argument', () => {
        const argumentTest = new FfmpegMediaFile(inputQuicktimePath);
        argumentTest.addOption('ss', 60);
        expect(argumentTest.ffmpegSettings.options).to.deep.equal({ ss: 60 });
      });

      it('returns this', () => {
        const argumentTest = new FfmpegMediaFile(inputQuicktimePath);
        expect(argumentTest.addOption('vn')).to.equal(argumentTest);
      });
    });

    describe('hasOption()', () => {
      it('returns false when option does not exist', () => {
        const testVal = new FfmpegMediaFile(inputQuicktimePath);
        expect(testVal.ffmpegSettings.options).to.deep.equal({}, 'Options were not empty immediately after construction');
        testVal.addOption('sn');
        expect(testVal.hasOption('-c:v')).to.be.false; // eslint-disable-line no-unused-expressions
      });

      it('returns true when option exists', () => {
        const testVal = new FfmpegMediaFile(inputQuicktimePath);
        expect(testVal.ffmpegSettings.options).to.deep.equal({}, 'Options were not empty immediately after construction');
        testVal.vn();
        expect(testVal.hasOption('vn')).to.be.true; // eslint-disable-line no-unused-expressions
      });
    });

    describe('removeOption()', () => {
      it('removes an argument', () => {
        const argumentTest = new FfmpegMediaFile(inputQuicktimePath);
        argumentTest.addOption('ss', 60);
        expect(argumentTest.ffmpegSettings.options).to.deep.equal({ ss: 60 }, 'Failed to set initial values to be deleted');
        argumentTest.removeOption('ss');
        expect(argumentTest.ffmpegSettings.options).to.deep.equal({});
      });

      it('returns this', () => {
        const argumentTest = new FfmpegMediaFile(inputQuicktimePath);
        argumentTest.addOption('ss', 60);
        expect(argumentTest.removeOption('ss')).to.equal(argumentTest);
      });
    });

    describe('getCommandLine()', () => {
      it('returns an array', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath).getCommandLine();
        expect(test).to.be.an('array');
      });

      it('returns empty array if no added options', () => {
        const testArguments = new FfmpegMediaFile(inputQuicktimePath).getCommandLine();
        expect(testArguments).to.deep.equal([]);
      });

      it('returns input options for one added option', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.addOption('ss', 60);
        expect(test.getCommandLine()).to.deep.equal(['-ss', 60]);
      });

      it('returns input options and with three added arguments', () => {
        const test = new FfmpegMediaFile(inputQuicktimePath);
        test.addOption('ss', 60);
        test.addOption('vn');
        test.addOption('-c:v', 'prores');
        expect(test.getCommandLine()).to.deep.equal(['-ss', 60, '-vn', '-c:v', 'prores']);
      });
    });
  });
});
