function parseNumber(number) {
  if (Array.isArray(number)) {
    return number.map(parseNumber);
  }

  if (typeof number === 'object') {
    const obj = {};
    Object.keys(number).forEach((key) => {
      obj[key] = parseNumber(number[key]);
    });

    return obj;
  }

  if (/^\d+\.\d+$/.test(number)) {
    return parseFloat(number, 10);
  }

  if (/^\d+$/.test(number)) {
    return parseInt(number, 10);
  }

  return number;
}

class FfmpegParsedData {
  constructor(data) {
    if (data) {
      const copy = JSON.parse(
        JSON.stringify(
          parseNumber(data),
        ),
      );
      copy.streams.sort((a, b) => a.index - b.index);
      Object.assign(this, copy);
    } else {
      this.format = {};
      this.streams = [];
    }

    this.freezeParsedData();
  }

  freezeParsedData(data = this) {
    const properties = Object.getOwnPropertyNames(data);

    properties.forEach((key) => {
      if (typeof data[key] === 'object') {
        this.freezeParsedData(data[key]);
      }
    });

    Object.freeze(data);
  }

  get(stringify = false) {
    const json = {
      format: this.format,
      streams: this.streams,
    };

    if (stringify) {
      return JSON.stringify(json);
    }

    return json;
  }
}

module.exports = FfmpegParsedData;
