const Helpers = require('./FfmpegProgram.helpers');

const ffmpegVersion = Helpers.parseFfmpegVersionNumber();
const buildConfiguration = Helpers.parseFfmpegBuildOutput();
const { ffmpegVideoEncoders, ffmpegAudioEncoders } = Helpers.parseFfmpegEncoderOutput();
const { ffmpegVideoDecoders, ffmpegAudioDecoders } = Helpers.parseFfmpegDecoderOutput();
const ffmpegFilters = Helpers.parseFfmpegFilterOutput();
const ffmpegVideoFilters = Object.keys(ffmpegFilters).filter(code => ffmpegFilters[code].input === 'video' || ffmpegFilters[code].output === 'video');
const ffmpegAudioFilters = Object.keys(ffmpegFilters).filter(code => ffmpegFilters[code].input === 'audio' || ffmpegFilters[code].output === 'audio');
const ffmpegSampleFmts = Helpers.parseFfmpegSampleFmtsOutput();

const FfmpegProgram = {
  getVersion: function getVersion() {
    return ffmpegVersion;
  },

  getBuildConfiguration: function getBuildConfiguration() {
    return buildConfiguration;
  },

  getSupportedResizePresets: function getSupportedResizePresets() {
    return Object.keys(Helpers.FfmpegResizePresets);
  },

  getResizePreset: function getResizePreset(preset) {
    return Helpers.FfmpegResizePresets[preset];
  },

  getSupportedVideoEncoders: function getSupportedVideoEncoders() {
    return Object.keys(ffmpegVideoEncoders);
  },

  getSupportedVideoDecoders: function getSupportedVideoDecoders() {
    return Object.keys(ffmpegVideoDecoders);
  },

  getSupportedAudioEncoders: function getSupportedAudioEncoders() {
    return Object.keys(ffmpegAudioEncoders);
  },

  getSupportedAudioDecoders: function getSupportedAudioDecoders() {
    return Object.keys(ffmpegAudioDecoders);
  },

  getSupportedFilters: function getSupportedFilters() {
    return Object.keys(ffmpegFilters);
  },

  getSupportedVideoFilters: function getSupportedVideoFilters() {
    return ffmpegVideoFilters;
  },

  getSupportedAudioFilters: function getSupportedAudioFilters() {
    return ffmpegAudioFilters;
  },

  getSupportedSampleFmts: function getSupportedSampleFmts() {
    return ffmpegSampleFmts;
  },

  getFilter: function getFilter(filter) {
    return ffmpegFilters[filter];
  },
};

module.exports = FfmpegProgram;
