/* eslint-env mocha */
const { expect } = require('chai');
const FfmpegProgram = require('./FfmpegProgram');
require('dotenv').config();

describe('FfmpegProgram', () => {
  describe('getVersion()', () => {
    it('returns a number', () => {
      const testVal = FfmpegProgram.getVersion();
      expect(testVal).to.be.a('number');
    });
  });

  describe('getBuildConfiguration', () => {
    it('returns an object', () => {
      const testVal = FfmpegProgram.getBuildConfiguration();
      expect(testVal).to.be.an('object');
    });
  });

  describe('getSupportedVideoEncoders()', () => {
    it('returns an Array', () => {
      const testVal = FfmpegProgram.getSupportedVideoEncoders();
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      const testVal = FfmpegProgram.getSupportedVideoEncoders();
      expect(testVal.length).to.not.equal(0);
    });

    it('has all encoder names in proper format', () => {
      const testVal = FfmpegProgram.getSupportedVideoEncoders();
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^(?:\w|-)+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedVideoDecoders()', () => {
    const testVal = FfmpegProgram.getSupportedVideoDecoders();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all encoder names in proper format', () => {
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^(?:\w|-)+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedAudioEncoders()', () => {
    const testVal = FfmpegProgram.getSupportedAudioEncoders();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all encoder names in proper format', () => {
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^(?:\w|-)+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedAudioDecoders()', () => {
    const testVal = FfmpegProgram.getSupportedAudioDecoders();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all encoder names in proper format', () => {
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^(?:\w|-)+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedFilters()', () => {
    const testVal = FfmpegProgram.getSupportedFilters();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all encoder names in proper format', () => {
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^(?:\w|-)+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedVideoFilters()', () => {
    const testVal = FfmpegProgram.getSupportedVideoFilters();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all filter names in proper format', () => {
      testVal.forEach((code) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^\w+$/.test(code), `${code} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getSupportedAudioFilters()', () => {
    const testVal = FfmpegProgram.getSupportedAudioFilters();

    it('returns an Array', () => {
      expect(testVal).to.be.an('array');
    });

    it('does not return an empty Array', () => {
      expect(testVal.length).to.not.equal(0);
    });

    it('has all filter names in proper format', () => {
      testVal.forEach((codec) => {
        // eslint-disable-next-line no-unused-expressions
        expect(/^\w+$/.test(codec), `${codec} did not match regex pattern`).to.be.true;
      });
    });
  });

  describe('getFilter()', () => {
    const nullsrc = FfmpegProgram.getFilter('nullsrc');
    const nullsink = FfmpegProgram.getFilter('nullsink');
    const threshold = FfmpegProgram.getFilter('threshold');
    const scale2ref = FfmpegProgram.getFilter('scale2ref');
    const afir = FfmpegProgram.getFilter('afir');

    it('returns an object for valid filter', () => {
      expect(nullsrc).to.be.an('object');
    });

    it('returns a description for valid filter', () => {
      expect(nullsrc.description).to.equal('Null video source, return unprocessed video frames.');
    });

    it('returns "null" for input of a filter with a null input', () => {
      expect(nullsrc.input).to.equal('null');
    });

    it('returns "null" for output of a filter with a null output', () => {
      expect(nullsink.output).to.equal('null');
    });

    it('returns "video" for input of a filter with a video input', () => {
      expect(nullsink.input).to.equal('video');
    });

    it('returns "video" for output of a filter with video output', () => {
      expect(nullsrc.output).to.equal('video');
    });

    it('returns "video" for output of a filter with multiple video outputs', () => {
      expect(scale2ref.output).to.equal('video');
    });
  });
});
