/* eslint camelcase: "off" */
// Adds a hyphen to the beginning of an option, if it doesn't already have one
function checkAndRemoveHyphenFromOption(option) {
  return option[0] === '-' ? option.substring(1) : option;
}

function formatOptionToObject(option, setting) {
  const key = checkAndRemoveHyphenFromOption(option);
  const value = setting === undefined ? null : setting;

  return { [key]: value };
}

module.exports = class FfmpegSettings {
  constructor() {
    this.options = {};
    this.customMetadata = {};
    this.filters = [];
  }

  metadata(key, value) {
    if (typeof key === 'object') {
      Object.assign(this.customMetadata, JSON.parse(JSON.stringify(key)));
    }

    if (key && value) {
      this.customMetadata[key] = value;
    }
  }

  hasOption(option) {
    if (option === undefined) {
      return false;
    }

    const sanitizedOption = checkAndRemoveHyphenFromOption(option);
    return Object.prototype.hasOwnProperty.call(this.options, sanitizedOption);
  }

  addOption(option, setting) {
    if (!option) {
      return;
    }

    const obj = formatOptionToObject(option, setting);
    Object.assign(this.options, obj);
  }

  removeOption(option) {
    if (!option || !this.hasOption(option)) {
      return;
    }

    const sanitizedOption = checkAndRemoveHyphenFromOption(option);
    delete this.options[sanitizedOption];
  }

  getMetadataCommandLine(streamSpecifier = '') {
    const separator = streamSpecifier === '' ? '' : ':';

    const commandLineOptions = [];
    Object.keys(this.customMetadata).forEach((key) => {
      commandLineOptions.push(
        `-metadata${separator}${streamSpecifier}`,
        `${key}="${this.customMetadata[key]}"`,
      );
    });

    return commandLineOptions;
  }

  getOptionsCommandLine(streamSpecifier = '') {
    const separator = streamSpecifier === '' ? '' : ':';

    const commandLineOptions = [];
    Object.keys(this.options).forEach((key) => {
      if (this.options[key] === null) {
        commandLineOptions.push(`-${key}`);
      } else {
        commandLineOptions.push(
          `-${key}${separator}${streamSpecifier}`, this.options[key],
        );
      }
    });

    return commandLineOptions;
  }

  getCommandLine() {
    return [].concat(
      this.getOptionsCommandLine(),
      this.getMetadataCommandLine(),
    );
  }
};
