/* eslint-env mocha */
const { expect } = require('chai');
const FfmpegStreamFactory = require('./FfmpegStreamFactory.js');
const FfmpegStream = require('../FfmpegStream/FfmpegStream');
const FfmpegStreamVideoInput = require('../FfmpegStreamVideoInput/FfmpegStreamVideoInput');
const FfmpegStreamAudioInput = require('../FfmpegStreamAudioInput/FfmpegStreamAudioInput');
const FfmpegStreamVideoOutput = require('../FfmpegStreamVideoOutput/FfmpegStreamVideoOutput');
const FfmpegStreamAudioOutput = require('../FfmpegStreamAudioOutput/FfmpegStreamAudioOutput');
const { inputFileMock } = require('../../test/fixtures');

describe('FfmpegStreamFactory', () => {
  describe('InputStreamFactory', () => {
    it('it creates an FfmpegStreamVideoInput with "codec_type": "video"', () => {
      const test = FfmpegStreamFactory.InputStreamFactory(inputFileMock.parsedData.streams[1]);
      expect(test).to.be.an.instanceOf(FfmpegStreamVideoInput);
    });

    it('it creates an FfmpegStreamVideoInput with "codec_type": "video"', () => {
      const test = FfmpegStreamFactory.InputStreamFactory(inputFileMock.parsedData.streams[0]);
      expect(test).to.be.an.instanceOf(FfmpegStreamAudioInput);
    });

    it('it creates an FfmpegStream with "codec_type": "data"', () => {
      const test = FfmpegStreamFactory.InputStreamFactory(inputFileMock.parsedData.streams[2]);
      expect(test).to.be.an.instanceOf(FfmpegStream);
    });
  });

  describe('OutputStreamFactory', () => {
    it('it creates an FfmpegStreamVideoOutput with "codec_type": "video"', () => {
      const rawStream = inputFileMock.parsedData.streams[1];
      expect(rawStream.codec_type, 'Test fixture is not a video stream').to.equal('video');
      const test = FfmpegStreamFactory.OutputStreamFactory(inputFileMock.parsedData.streams[1]);
      expect(test).to.be.an.instanceOf(FfmpegStreamVideoOutput);
    });

    it('it creates an FfmpegStreamAudioOutput with "codec_type": "audio"', () => {
      const rawStream = inputFileMock.parsedData.streams[0];
      expect(rawStream.codec_type, 'Test fixture is not an audio stream').to.equal('audio');
      const test = FfmpegStreamFactory.OutputStreamFactory(rawStream);
      expect(test).to.be.an.instanceOf(FfmpegStreamAudioOutput);
    });

    it('it creates an FfmpegStream with "codec_type": "data"', () => {
      const test = FfmpegStreamFactory.OutputStreamFactory(inputFileMock.parsedData.streams[2]);
      expect(test).to.be.an.instanceOf(FfmpegStream);
    });
  });
});
