/* eslint camelcase: "off" */
const FfmpegStream = require('../FfmpegStream/FfmpegStream');

module.exports = class FfmpegStreamVideo extends FfmpegStream {
  /**
   * Set pixel format. Use -pix_fmts to show all the supported pixel formats. If the selected pixel
   * format can not be selected, ffmpeg will print a warning and select the best pixel format
   * supported by the encoder. If pix_fmt is prefixed by a +, ffmpeg will exit with an error if the
   * requested pixel format can not be selected, and automatic conversions inside filtergraphs are
   * disabled. If pix_fmt is a single +, ffmpeg selects the same pixel format as the input (or graph
   * output) and automatic conversions are disabled.
   *
   * @param format
   */
  pix_fmt(format) {
    return this.addOption('-pix_fmt', format);
  }

  /**
   * Set frame rate (Hz value, fraction or abbreviation).
   * As an input option, ignore any timestamps stored in the file and instead generate timestamps
   * assuming constant frame rate fps. This is not the same as the -framerate option used for some
   * input formats like image2 or v4l2 (it used to be the same in older versions of FFmpeg). If in
   * doubt use -framerate instead of the input option -r.
   *
   * As an output option, duplicate or drop input frames to achieve constant output frame rate fps.
   *
   * @param {number} r
   */
  r(fps) {
    return this.ffmpegSettings.addOption('-r', fps);
  }

  /**
   * Set frame size.
   * As an input option, this is a shortcut for the video_size private option, recognized by some
   * demuxers for which the frame size is either not stored in the file or is configurable – e.g.
   * raw video or video grabbers.
   *
   * As an output option, this inserts the scale video filter to the end of the corresponding
   * filtergraph. Please use the scale filter directly to insert it at the beginning or some other
   * place.
   *
   * @param {string} size In the format wxh
   */
  s(size) {
    return this.ffmpegSettings.addOption('-s', size);
  }
};
