const FFMPEG_TIME_REGEX = /^-?(?:\d+:)?(?:\d{1,2}):(?:\d{1,2})(?:\.\d+)?$/;
const ZERO_STRING_REGEX = /^-?(?:0+:)?(?:0{1,2}):(?:0{1,2})(?:\.0+)?$/;

function toInteger(arg) {
  const asInt = Number.parseInt(arg, 10);
  if (asInt == arg) { // eslint-disable-line eqeqeq
    return asInt;
  }

  throw new TypeError(`${arg} could not be converted to an integer`);
}

function toFloat(arg) {
  const asFloat = Number.parseFloat(arg);
  if (asFloat == arg) { // eslint-disable-line eqeqeq
    return asFloat;
  }

  throw new TypeError(`${arg} could not be converted to a float`);
}

class FfmpegTime {
  constructor(time) {
    this.set(time);
  }

  isValid() {
    return this.get() !== undefined;
  }

  isNegative() {
    if (!this.isValid()) {
      return false;
    }

    if (!Number.isNaN(this.get()) && this.get() < 0) {
      return true;
    }

    if (this.get()[0] === '-' && !ZERO_STRING_REGEX.test(this.get())) {
      return true;
    }

    return false;
  }

  isPositive() {
    return (this.isValid() && !this.isNegative() && !this.isZero());
  }

  isZero() {
    return (this.isValid() && (this.get() === 0 || ZERO_STRING_REGEX.test(this.get())));
  }

  get() {
    return this.value;
  }

  set(time) {
    if (FFMPEG_TIME_REGEX.test(time)) {
      this.value = time;
      return;
    }

    const asFloat = parseFloat(time);
    if (!Number.isNaN(asFloat) && asFloat == time) { // eslint-disable-line eqeqeq
      this.value = asFloat;
    }
  }
}

module.exports = {
  FfmpegTime,
  toInteger,
  toFloat,
};
