/* eslint-env mocha */
/* eslint-disable no-unused-expressions */
const { expect } = require('chai');
const { FfmpegTime } = require('./FfmpegValidators');

describe('FfmpegTime', () => {
  describe('Constructor', () => {
    it('accepts an integer argument', () => {
      const test = new FfmpegTime(60);
      expect(test.value).to.equal(60);
    });

    it('accepts a float argument', () => {
      const test = new FfmpegTime(34.67);
      expect(test.value).to.equal(34.67);
    });

    it('accepts an integer stored as string argument', () => {
      const test = new FfmpegTime('60');
      expect(test.value).to.equal(60);
    });

    it('accepts a float stored as string argument', () => {
      const test = new FfmpegTime('34.67');
      expect(test.value).to.equal(34.67);
    });

    it('accepts an mm:ss argument', () => {
      const test = new FfmpegTime('25:15');
      expect(test.value).to.equal('25:15');
    });

    it('accepts an h:mm:ss argument', () => {
      const test = new FfmpegTime('0:25:15');
      expect(test.value).to.equal('0:25:15');
    });

    it('accepts an hh:mm:ss.mm argument', () => {
      const test = new FfmpegTime('0:25:15.125');
      expect(test.value).to.equal('0:25:15.125');
    });

    it('remains undefined called with an invalid time string', () => {
      const test = new FfmpegTime('1:5.6.7');
      expect(test.value).to.be.undefined;
    });
  });

  describe('isNegative()', () => {
    it('returns false for a positive integer', () => {
      const test = new FfmpegTime(60);
      expect(test.isNegative()).to.be.false;
    });

    it('returns false for a positive time string', () => {
      const test = new FfmpegTime('0:25:15.125');
      expect(test.isNegative()).to.be.false;
    });

    it('returns true for a negative time string', () => {
      const test = new FfmpegTime('-25:15');
      expect(test.isNegative()).to.be.true;
    });

    it('returns true for a negative integer', () => {
      const test = new FfmpegTime(-45);
      expect(test.isNegative()).to.be.true;
    });

    it('returns false for zero', () => {
      const test = new FfmpegTime(0);
      expect(test.isNegative()).to.be.false;
    });

    it('returns false for a zero time string', () => {
      const test = new FfmpegTime('00:00');
      expect(test.isNegative()).to.be.false;
    });

    it('returns false for invalid value', () => {
      const test = new FfmpegTime('foo');
      expect(test.isNegative()).to.be.false;
    });
  });

  describe('isPositive()', () => {
    it('returns true for a positive integer', () => {
      const test = new FfmpegTime(60);
      expect(test.isPositive()).to.be.true;
    });

    it('returns true for a positive time string', () => {
      const test = new FfmpegTime('0:25:15.125');
      expect(test.isPositive()).to.be.true;
    });

    it('returns false for a negative time string', () => {
      const test = new FfmpegTime('-25:15');
      expect(test.isPositive()).to.be.false;
    });

    it('returns false for a negative integer', () => {
      const test = new FfmpegTime(-45);
      expect(test.isPositive()).to.be.false;
    });

    it('returns false for zero', () => {
      const test = new FfmpegTime(0);
      expect(test.isPositive()).to.be.false;
    });

    it('returns false for a zero time string', () => {
      const test = new FfmpegTime('00:00');
      expect(test.isPositive()).to.be.false;
    });

    it('returns false for invalid value', () => {
      const test = new FfmpegTime('foo');
      expect(test.isPositive()).to.be.false;
    });
  });

  describe('isZero()', () => {
    it('returns false for a positive integer', () => {
      const test = new FfmpegTime(60);
      expect(test.isZero()).to.be.false;
    });

    it('returns false for a positive time string', () => {
      const test = new FfmpegTime('0:25:15.125');
      expect(test.isZero()).to.be.false;
    });

    it('returns false for a negative time string', () => {
      const test = new FfmpegTime('-25:15');
      expect(test.isZero()).to.be.false;
    });

    it('returns true for a negative integer', () => {
      const test = new FfmpegTime(-45);
      expect(test.isZero()).to.be.false;
    });

    it('returns true for zero', () => {
      const test = new FfmpegTime(0);
      expect(test.isZero()).to.be.true;
    });

    it('returns true for a zero time string', () => {
      const test = new FfmpegTime('00:00');
      expect(test.isZero()).to.be.true;
    });

    it('returns false for invalid value', () => {
      const test = new FfmpegTime('foo');
      expect(test.isZero()).to.be.false;
    });
  });
});
